@@ -526,6 +526,10 @@ static int smtp_get_effective_tls_level(DSN_BUF *why, SMTP_STATE *state)
     switch (state->reqtls_level) {
     case SMTP_REQTLS_POLICY_ACT_ENFORCE:
 	if (TLS_MUST_MATCH(tls->level) == 0) {
+	    if (state->tls_stats)
+		smtp_tls_stat_decide_reqtls(state->tls_stats,
+					    TLS_STAT_VIOLATION,
+					    TLS_STAT_ENF_FULL);
 	    dsb_simple(why, "5.7.10", "REQUIRETLS Failure: sender "
 		       "requested REQUIRETLS, but my configured TLS "
 		       "security level '%s' disables certificate "
@@ -536,6 +540,10 @@ static int smtp_get_effective_tls_level(DSN_BUF *why, SMTP_STATE *state)
 	break;
     case SMTP_REQTLS_POLICY_ACT_OPP_TLS:
 	if (tls->level == TLS_LEV_NONE) {
+	    if (state->tls_stats)
+		smtp_tls_stat_decide_reqtls(state->tls_stats,
+					    TLS_STAT_VIOLATION,
+					    TLS_STAT_ENF_RELAXED);
 	    dsb_simple(why, "5.7.10", "REQUIRETLS Failure: sender "
 		       "requested REQUIRETLS, but my configured TLS "
 		       "security level '%s' disables encryption. The "
@@ -1187,6 +1195,15 @@ static void smtp_connect_inet(SMTP_STATE *state, const char *nexthop,
 		continue;
 		/* XXX Assume there is no code at the end of this loop. */
 	    }
+	    if (state->tls_stats) {
+		tls_stats_revert(state->tls_stats);
+		smtp_tls_stat_activate_sec_level(state->tls_stats,
+						 state->tls->level,
+					  state->tls->level != TLS_LEV_MAY);
+		if (state->reqtls_level > SMTP_REQTLS_POLICY_ACT_DISABLE)
+		    smtp_tls_stat_activate_reqtls(state->tls_stats,
+		     state->reqtls_level == SMTP_REQTLS_POLICY_ACT_ENFORCE);
+	    }
 	    /* Disable TLS when retrying after a handshake failure */
 	    if (retry_plain) {
 		state->tls->level = TLS_LEV_NONE;
